@@ -52,6 +52,7 @@ class MassGrid:
                     (jj[0], jj[1], ii)      # m/z, masstrack_id, sample_index
                 )
         all.sort()
+
         all_bins = self.bin_track_mzs(all, self.experiment.reference_sample_id)
 
         self.MassGrid = pd.DataFrame(
@@ -135,7 +136,7 @@ class MassGrid:
 
     def bin_track_mzs(self, tl, reference_id):
         '''
-        Bin all track m/z values into centroids, to be used to build massGrid.
+        Bin all track m/z values into centroids via clustering, to be used to build massGrid.
         Because the range of each bin cannot be larger than mz_tolerance, 
         and mass tracks in each sample cannot overlap within mz_tolerance,
         multiple entries from the same sample in same bin will not happen.
@@ -150,10 +151,9 @@ class MassGrid:
         list of bins: [ (mean_mz, [(), (), ...]), (mean_mz, [(), (), ...]), ... ]
         '''
         def __get_bin__(bin_data_tuples):
-            return (np.mean([x[0] for x in bin_data_tuples]), bin_data_tuples)
+            return (np.median([x[0] for x in bin_data_tuples]), bin_data_tuples)
 
         tol_ = 0.000001 * self.experiment.parameters['mz_tolerance_ppm']
-
         bins_of_bins = []
         tmp = [tl[0]]
         for ii in range(1, len(tl)):
@@ -165,36 +165,20 @@ class MassGrid:
                 bins_of_bins.append(tmp)
                 tmp = [tl[ii]]
         bins_of_bins.append(tmp)
-
         good_bins = []
         for bin_data_tuples in bins_of_bins:
             mz_range = bin_data_tuples[-1][0] - bin_data_tuples[0][0]
             mz_tolerance = bin_data_tuples[0][0] * tol_
-            if mz_range < mz_tolerance:
+            # important: double tol_ range here as mean_mz falls in single tol_
+            if mz_range < mz_tolerance * 2:
                 good_bins.append( __get_bin__(bin_data_tuples) )
 
             else:
-                clusters, remaining = [], []                     # need complete asignment form MassGrid
-                num_steps = int( 5* mz_range/ mz_tolerance )     # step in 1/5 mz_tolerance
-                hist, bin_edges = np.histogram([x[0] for x in bin_data_tuples], num_steps)
-                # example hist: array([  8,   33,  11,  24,  31,  50,  81, 164, 269,  28,   7])
-                hist_starts = [0] + [hist[:ii].sum() for ii in range(1,num_steps+1)]
-                # find_peaks returns e.g. array([ 1, 8]), {}. distance=5 because it's edge of tolerance
-                peaks, _ = find_peaks( hist, distance = 5 )
-                if peaks.any():
-                    clusters = seed_nn_mz_cluster(bin_data_tuples, [hist_starts[ii] for ii in peaks])
-                    for clu in clusters:
-                        if clu[-1][0] - clu[0][0] < mz_tolerance:
-                            good_bins.append( __get_bin__(clu) )
-                        else:
-                            remaining += clu
-                    good_bins += [__get_bin__(C) for C in gap_divide_mz_cluster(remaining, mz_tolerance)]
-
-                else:
-                    good_bins += [__get_bin__(C) for C in gap_divide_mz_cluster(bin_data_tuples, mz_tolerance)]
+                good_bins += [__get_bin__(C) for C in nn_cluster_by_mz_seeds(bin_data_tuples, mz_tolerance)]
 
         return good_bins
 
+
     def join(self, M2):
         '''
         Join with another MassGridCluster. Placeholder.
@@ -210,19 +194,6 @@ class CompositeMap:
     i) MassGrid: a matrix (DataFrame) for correspondence of mass tracks to each sample 
     ii) FeatureList: list of feature definitions, i.e. peaks defined on composite mass tracks.
     iii) FeatureTable: a matrix for feature intensities per sample
-
-    Steps:
-    1. Build MassGrid. Choose one reference from all samples for the largest number of landmark m/z tracks.
-    2. Add each of remaining samples to MassGrid, 
-    m/z alignment iguided by matched isotope/adduct pairs. Reference m/z is updated every time.
-    RT alignment function is determined per sample, using selective landmark peaks.
-    Default is a LOWESS function, but open to others to plugin.
-    3. Build composite elution profile (composite_mass_tracks)
-    by cumulative sum of mass tracks from all samples after RT correction.
-    4. Global peak detection is performed on each composite massTrack.
-    5. Mapping global peaks (i.e. features) back to all samples and extract sample specific peak areas.
-    This completes the FeatureTable.
-    6. Grouping features to empirical compounds (defined in metDataModel package).
     '''
     def __init__(self, experiment):
         '''
@@ -316,8 +287,9 @@ class CompositeMap:
             for k in mzlist:
                 ref_index = self.MassGrid[SM.name][k]
                 if not pd.isna(ref_index): # ref_index can be NA 
-                    _comp_dict[k] += remap_intensity_track(
-                        list_mass_tracks[int(ref_index)]['intensity'], basetrack.copy(), SM.rt_cal_dict)
+                    _comp_dict[k] += remap_intensity_track( list_mass_tracks[int(ref_index)]['intensity'],  
+                                                            basetrack.copy(), SM.rt_cal_dict )
+
         result = {}
         for k,v in _comp_dict.items():
             result[k] = { 'id_number': k, 'mz': mzDict[k], 'intensity': v }
@@ -477,20 +449,15 @@ class CompositeMap:
 
 
 
-
-
-
 class SampleAssembly(CompositeMap):
     '''
     
-    
     Alternative workflow:
     1) construct MassGrid
     2) peak detection in each sample, 
     3) RT align by distance threshold.
 
 
-
     '''
 
 
